PetBattle = select(2, ...)
PetBattle.modules = {}
PetBattle.MAX_PETS = NUM_BATTLE_PETS_IN_BATTLE or 3
PetBattle.MAX_ABILITIES = NUM_BATTLE_PET_ABILITIES or 3

local L = PetBattle.L

local battleChangedEvents = {"PET_BATTLE_PET_CHANGED", "PET_BATTLE_TURN_STARTED", "PET_BATTLE_PET_ROUND_PLAYBACK_COMPLETE"}
function PetBattle:OnInitialize()
	if( not PetBattleTacticianDB ) then PetBattleTacticianDB = {showVs = true, showAbilities = true} end
	self.db = PetBattleTacticianDB

	-- Alias the events taht require us to update
	for _, event in pairs(battleChangedEvents) do
		self:RegisterEvent(event)
		self[event] = self.Update
	end

	-- Initial ones we always care about
	self:RegisterEvent("PET_BATTLE_OPENING_START")
	self:RegisterEvent("PET_BATTLE_CLOSE")

	-- And in case we reloaded 
	if( C_PetBattles.IsInBattle() ) then
		self:PET_BATTLE_OPENING_START()
	end
end

function PetBattle:PET_BATTLE_OPENING_START()
	self.battleSetup = true
	self:TriggerModules("Create")
	self:Update()
end

function PetBattle:PET_BATTLE_CLOSE()
	self:TriggerModules("Reset")
	self.battleSetup = nil
end

function PetBattle:Update()
	-- PET_BATTLE_PET_CHANGED fires before PET_BATTLE_OPENING_DONE
	if( not self.battleSetup ) then return end

	local battlePets = {[LE_BATTLE_PET_ALLY] = {}, [LE_BATTLE_PET_ENEMY] = {}}

	for ownerType, pets in pairs(battlePets) do
		local activeSlot = C_PetBattles.GetActivePet(ownerType)

		for petSlot=1, C_PetBattles.GetNumPets(ownerType) do
			local speciesID = C_PetBattles.GetPetSpeciesID(ownerType, petSlot)
			if( speciesID ) then
				local name, icon, petType, creatureID, sourceText, description, isWild, canBattle, tradable, unique = C_PetJournal.GetPetInfoBySpeciesID(speciesID)

				local pet = {["name"] = name, ["icon"] = icon, ["petType"] = petType, ["speciesID"] = speciesID}
				pet["icon"] = C_PetBattles.GetIcon(ownerType, petSlot)
				pet["active"] = activeSlot == petSlot

				-- Figure out the stats
				pet["quality"] = C_PetBattles.GetBreedQuality(ownerType, petSlot)
				pet["level"] = C_PetBattles.GetLevel(ownerType, petSlot)
				pet["speed"] = C_PetBattles.GetSpeed(ownerType, petSlot)
				pet["power"] = C_PetBattles.GetPower(ownerType, petSlot)
				pet["health"] = C_PetBattles.GetHealth(ownerType, petSlot)
				pet["maxhealth"] = C_PetBattles.GetMaxHealth(ownerType, petSlot)

				-- Load abilities
				pet["abilities"] = {}
				for abilitySlot=1, NUM_BATTLE_PET_ABILITIES do
					local id, name, icon, maxCooldown, description, numTurns, abilityPetType, noStrongWeakHints = C_PetBattles.GetAbilityInfo(ownerType, petSlot, abilitySlot);
					if( id ) then
						local ability = {["id"] = id, ["name"] = name, ["icon"] = icon, ["petType"] = abilityPetType, ["hints"] = not noStrongWeakHints}
						ability["usable"], ability["cooldown"] = C_PetBattles.GetAbilityState(ownerType, petSlot, abilitySlot)

						pet["abilities"][abilitySlot] = ability
					end
				end

				pets[petSlot] = pet
			end
		end
	end

	self:TriggerModules("Update", battlePets)
end

-- Tooltips
function PetBattle.ShowPetTooltip(self, anchor)
	PetBattleUnitTooltip_Attach(PetBattlePrimaryUnitTooltip, "TOPLEFT", anchor or self, "BOTTOMLEFT", -4, -10)
	PetBattleUnitTooltip_UpdateForUnit(PetBattlePrimaryUnitTooltip, self.petOwner, self.petSlot)
	PetBattlePrimaryUnitTooltip:Show()
end

function PetBattle.HidePetTooltip(self)
	PetBattlePrimaryUnitTooltip:Hide()
end

function PetBattle.ShowAbilityTooltip(self)
	PetBattleAbilityTooltip_SetAbility(self.petOwner, self.petSlot, self.petAbilitySlot)
	PetBattleAbilityTooltip_Show("TOPLEFT", self, "TOPLEFT", -4, -24)
end

function PetBattle.HideAbilityTooltip(self)
	PetBattlePrimaryAbilityTooltip:Hide()
end

function PetBattle.HideTooltip(self)
	GameTooltip:Hide()
end

-- Module management
function PetBattle:RegisterModule(type)
	self.modules[type] = {}
	return self.modules[type]
end

function PetBattle:TriggerModules(type, ...)
	for _, obj in pairs(self.modules) do
		obj[type](obj, ...)
	end
end

-- Slash commands
function PetBattle:Print(msg)
	DEFAULT_CHAT_FRAME:AddMessage("|cff33ff99Pet Battle Tactician|r: " .. msg)
end

SLASH_PETBATTLETACT1 = "/pbt"
SLASH_PETBATTLETACT2 = "/petbattletact"
SLASH_PETBATTLETACT3 = "/petbattletactician"
SlashCmdList["PETBATTLETACT"] = function(msg)
	msg = msg and string.lower(msg) 

	if( msg == "vs" ) then
		PetBattle.db.showVs = not PetBattle.db.showVs
		PetBattle:Print(string.format(L["Pet Vs frame is now %s"], PetBattle.db.showVs and L["on"] or L["off"]))


		if( PetBattle.db.showVs and C_PetBattles.IsInBattle() ) then
			PetBattle:PET_BATTLE_OPENING_DONE()
		elseif( not PetBattle.db.showVs ) then
			PetBattle:TriggerModules("SetStatus")
		end

	elseif( msg == "abilities" ) then
		PetBattle.db.showAbilities = not PetBattle.db.showAbilities
		PetBattle:Print(string.format(L["Abilities frame is now %s"], PetBattle.db.showAbilities and L["on"] or L["off"]))

		if( PetBattle.db.showAbilities and C_PetBattles.IsInBattle() ) then
			PetBattle:PET_BATTLE_OPENING_DONE()
		elseif( not PetBattle.db.showAbilities ) then
			PetBattle:TriggerModules("SetStatus")
		end

	else
		PetBattle:Print(L["Slash commands"])
		DEFAULT_CHAT_FRAME:AddMessage(L["/pbt abilities - Toggles the abilities frame on/off"])
		DEFAULT_CHAT_FRAME:AddMessage(L["/pbt vs - Toggles the pets vs frame on/off"])
	end
end


-- Event dispatch
local frame = CreateFrame("Frame")
frame:RegisterEvent("ADDON_LOADED")
frame:SetScript("OnEvent", function(self, event, addon)
	if( addon == "PetBattleTactician" ) then
		frame:UnregisterEvent("ADDON_LOADED")
		frame:SetScript("OnEvent", function(self, event, ...)
			PetBattle[event](PetBattle, event, ...)
		end)

		PetBattle:OnInitialize()
	end
end)

function PetBattle:RegisterEvent(event) frame:RegisterEvent(event) end
function PetBattle:UnregisterEvent(event) frame:UnregisterEvent(event) end
